#include <stdio.h>
#include <stdlib.h>
#include "Boolean.h"
#include "readField.h"
#include "tisane.h"
#include "materiePrime.h"

Boolean leggiComponente(FILE *f, Componente *c)
{
	return fscanf(f, "%d:%d", &c->materiaPrima, &c->percentuale) == 2;
}

Boolean componentePresente(FILE *f)
{
	return fgetc(f) == ';';
}

Boolean leggiTisana(FILE *f, Tisana *tisana)
{
	Boolean ok;
	ok = readField(f, tisana->nome, ';') == ';';
	ok = ok && fscanf(f, "%d", &tisana->materiaBase) == 1;

	tisana->dimComponenti = 0;
	while (ok && componentePresente(f))
	{
		ok = leggiComponente(f, tisana->componenti + tisana->dimComponenti);
		if (ok)
		{
			tisana->dimComponenti++;
		}
	}
	return ok;
}

void printAllProdotti(TisanaList list)
{
	while (list != NULL)
	{
		Tisana *tisana = &list->tisana;
		int i;
		printf("%s - Base: %d", tisana->nome, tisana->materiaBase);
		for (i = 0; i < tisana->dimComponenti; i++)
		{
			printf(" + [%d:%d]", tisana->componenti[i].materiaPrima, tisana->componenti[i].percentuale);
		}
		printf(" - Perc. Base: %d", calcolaQuantitaBase(*tisana));
		printf("\n");
		list = list->next;
	}
}

TisanaList appendTisana(Tisana t, TisanaList list)
{
	TisanaList newElement = (TisanaList)malloc(sizeof(TisanaListElement));
	newElement->tisana = t;
	newElement->next = NULL;
	if (list == NULL)
	{
		list = newElement;
	}
	else
	{
		TisanaList scan = list;
		while (scan->next != NULL)
		{
			scan = scan->next;
		}
		scan->next = newElement;
	}
	return list;
}

TisanaList leggiProdotti(char fileName[])
{
	TisanaList result = NULL;
	Tisana tisana;

	FILE *f = fopen(fileName, "r");
	if (f == NULL)
		abort();

	while (leggiTisana(f, &tisana))
	{
		result = appendTisana(tisana, result);
	}

	return result;
}

int calcolaQuantitaBase(Tisana tisana)
{
	int percBase = 100;
	int i;
	for (i = 0; i < tisana.dimComponenti; i++)
	{
		percBase -= tisana.componenti[i].percentuale;
	}
	return percBase;
}

float calcolaPrezzo(Tisana tisana, MateriaPrima mp[], int dim)
{
	float prezzo = 0;
	int materiaIndex;
	int i;
	MateriaPrima example;

	example.codice = tisana.materiaBase;
	materiaIndex = findElement(mp, dim, example);
	prezzo = mp[materiaIndex].prezzo * (float)calcolaQuantitaBase(tisana) / 100;

	for (i = 0; i < tisana.dimComponenti; i++)
	{
		example.codice = tisana.componenti[i].materiaPrima;
		materiaIndex = findElement(mp, dim, example);
		prezzo += mp[materiaIndex].prezzo * (float)tisana.componenti[i].percentuale / 100;
	}

	return prezzo;
}

void writeListino(FILE *f, TisanaList list, MateriaPrima mp[], int dim)
{
	while (list != NULL)
	{
		int i;
		int materiaIndex;
		MateriaPrima example;
		Tisana *tisana = &list->tisana;

		printf("%s: %.2f euro/Kg\n", tisana->nome, calcolaPrezzo(*tisana, mp, dim));

		example.codice = tisana->materiaBase;
		materiaIndex = findElement(mp, dim, example);
		printf("- %s: %d%%\n", mp[materiaIndex].nome, calcolaQuantitaBase(*tisana));
		for (i = 0; i < tisana->dimComponenti; i++)
		{
			example.codice = tisana->componenti[i].materiaPrima;
			materiaIndex = findElement(mp, dim, example);
			printf("- %s: %d%%\n", mp[materiaIndex].nome, tisana->componenti[i].percentuale);
		}
		printf("\n");
		list = list->next;
	}
}
